######################################
# Statistical methods for data science 
# 
# http://didawiki.di.unipi.it/doku.php/mds/smd/ 
######################################

# [CTRL-Enter] to execute a line

###### Script Lesson 07: R Data Access and Programming ##########

###### Data access ##########

# clean all
rm(list=ls())

# SET WORKING DIR FOR OUTPUT PDFs AND INPUT DATASETS (../../data)
setwd("C:/Users/Salvatore Ruggieri/Desktop/tmp")

# install packages
#install.packages("ISwR") # just once
library(ISwR) # attach (load) package
search() # attached packages (in RStudio/environment)

# visible data frames
data()

# a list of R datasets
# http://vincentarelbundock.github.io/Rdatasets/

# load from the web
iris.uci = read.csv(url("http://archive.ics.uci.edu/ml/machine-learning-databases/iris/iris.data"), header=FALSE)
colnames(iris.uci) = c("sepal.length","sepal.width","petal.length","petal.width","species")
View(iris.uci)

# load dataset
data(energy)

# view/edit/modifying data frame
View(energy)
energy2 = edit(energy) # edit - return modifications
fix(energy) # edit - modifications on the same data frame
xnew = edit(data.frame()) # new data frame from scratch

# I/O - see docs: R-data.pdf, including connection to RDBMS
write.table(energy, "file.data")  # csv format with row-id
write.csv(energy, "file.csv", row.names=FALSE)
e2 = read.table("file.data") 
e3 = read.csv("file.csv", encoding="UTF-8", sep=",", header=T) 
# working directory where file are saved/loaded
getwd() # get current working dir
save(energy, file="file.RData") # R binary format
rm(energy) # remove energy dataset
load("file.RData") # reload energy dataset
write(c("a", "b"), "file.txt")  # write vectors and matrices
save(list = ls(all = TRUE), file="file.RData") # R binary format
save.image("file.RData") # shortcut of previous line
rm(list=ls()) # remove all
load("file.RData") # reload workspace

# attach datasets
energy
attach(energy) 
expend # instead of energy$expend
detach('energy')
detach() # all datasets

# data summaries
attach(energy) 
mean(expend)
mean(c(1, 3, NA, 5), na.rm=T)
median(expend)
quantile(expend)
quantile(expend, seq(0, 1, 0.1))
sd(expend)
var(expend)
summary(energy)

energy[energy$expend < 10,] # sub-dataframe
subset(energy, expend<10) # sub-dataframe

# groups
split(expend, stature) 
# implicit loops
tapply(expend, stature, median)  # apply median on stature for each group of expend
tapply(stature, stature, length)  # or 
table(stature) # contingency table - one dimension

bins = cut(expend, c(5, 8, 11, 14)) # discretization in factors
bins
table(stature, bins) # contingency table - two dimensions

thuesen # dataset from ISwR
# implicit loops
lapply(thuesen, mean, na.rm=T) # list apply - apply median for each element of the list
sapply(thuesen, mean, na.rm=T) # simplify to a vector or matrix

sapply( split(expend, stature), mean) # this is tapply
# implicit loops on matrices
m = matrix(1:12, c(3,4))
m
apply( m, 2, sum) # second parameter 2 = cols, 1 = rows

# sorting and ranking
sort(expend, decreasing=FALSE)
o = order(expend)
o
expend[o]
energy[o,] # rows in order, all columns
rank(expend)
rank(c(1,2,2,4)) # see help(rank) for methods 
stature
unique(stature)

# plotting options
weight = c(60, 72, 57, 90, 95, 72)
height = c(1.75, 1.80, 1.65, 1.90, 1.74, 1.91)
plot(height, weight, pch=2, main="A sample plot", sub="about BMI", xlab="Height (m)", ylab="Weight (Kg)")

hh = c(1.65, 1.70, 1.75, 1.80, 1.85, 1.90)
lines(hh, 22.5 * hh^2) # or
curve(22.5 * x^2, add=T, col=2)

hist(expend) # how many bins?
hist(expend, breaks=4)
hist(expend, freq=F)

# discrete sampling - uniform
sample(c('a', 'b', 'c', 'd'), 5, replace=T)
sample(1:50, 5, replace=T)
hist(sample(1:50,10000,replace=T),freq=F) # 1000 -> 1000000

# discrete biased sampling
s = sample( c("H","T"), 1000, replace=T, prob=c(0.9, 0.1))
plot(as.factor(s)) # hist require numeric vector

# train-test splitting of energy
training_positions = sample(1:nrow(energy), nrow(energy)*.6)
training = energy[training_positions,] 
training
test = energy[-training_positions,]
test

# exercise: max of a vector

# sort then take last element
maxv0 = function(v) {
  v1 = sort(v)
  v1[length(v1)]
}

# iterate and keep maximum value so far
maxv1 = function(x) {
  m = -Inf # this implies sorting numbers!
  for(v in x)
    if(v > m)
      m = v
  return(m)
}

# iterate from 2 on and keep maximum value so far
maxv = function(x) {
  l = length(x)
  if(l==0)
    return(-Inf)
  m = x[1]
  for(v in x[-1]) # x[2:length(x)] wrong!
    if(v > m)
      m = v
  return(m)
}

maxv(c(1,2))
maxv(c(2))

# pay attention
c(1)[2:1]

# multiple assignment / unpacking of Python: NO!
#x1,y1 = 2,3 # error
x1=2; y1=3

# implicit loops
replicate(10, mean(rnorm(5))) # for 10 times: generate 5 random numbers and take the mean

# all and any
r = rnorm(100)
r
all(r>0)
any(r>0)

# compare maxv and max (built-in)
# install.packages("microbenchmark")
library(microbenchmark) # attach (load) package
microbenchmark( maxv(rnorm(10000)), times=1e3) 
microbenchmark( max(rnorm(10000)), times=1e3) # built-in's are faster

# factorial
fact = function(x) if(x<=0) 1 else x*fact(x-1)
sapply(1:20, fact)

# set working dir to current source file
# install.packages("rstudioapi")
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

# Reticulate: calling Python from R
# https://rstudio.github.io/reticulate/
# install.packages("reticulate")
library(reticulate)
source_python('smd07.py') # this may require to run RStudio in administrator-mode for installing required Python packages
add(5, 10)

# Reading Excel files
library(readxl)
# load table
dataxls = read_excel("smd07.xlsx", na = "n.a.")

# making factors
dataxls$Geography = as.factor(dataxls$Geography)
dataxls$State = as.factor(dataxls$State)
View(dataxls)

# pivoting
data_pivoted = data.frame()
for(i in 2010:2016) {
  # select a year
  data_year = dataxls[c("Geography", "State", paste("Year ", i, sep=""))]
  # rename year
  names(data_year)[3] = "Population"
  # add current year column
  data_year$Year = i
  data_year = data_year[, c(1,2,4,3)]
  # remove rows with all measures equal to NA
  data_year = data_year[ !is.na( data_year$Population ), ]
  # append
  data_pivoted = rbind(data_pivoted, data_year)
}

View(data_pivoted)

# Exercise at home: slide 37-38 of smd2.pdf
